import re

EXTERNAL = {
    "host": 'https://store.playstation.com',
    "product": 'https://store.playstation.com/ru-ru/product/',
    "ftp home": 'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-FREETOPLAYSEEALL/1',
    "ftp": 'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-FREETOPLAYSEEALL/%d',
    "soon home": 'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-COMINGSOON/1',
    "soon": 'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-COMINGSOON/%d',
    "top sellers home": 'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-GAMETOPSELLERS/1',
    "top sellers": 'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-GAMETOPSELLERS/%d',
    "store_homepage":
        'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-PS4CAT/1?platform=ps4',
    "new_games_homepage":
        'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-GAMELATEST/1?platform=ps4',
    "latest": 'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-GAMELATEST/%d',
    "all": 'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-PS4CAT/%d?platform=ps4',
    "ps plus": 'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-PLUSINSTANTGAME/1',
    "discounts": 'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-PRICEDROPSCHI/1?platform=ps4',
    "discounts homepage": 'https://store.playstation.com/ru-ru/grid/STORE-MSF75508-PRICEDROPSCHI/%d?platform=ps4',
}

SPECS = {
    "languages": ['немецкий', 'норвежский', 'финский', 'шведский',
                          'русский', 'португальский', 'английский', 'итальянский',
                          'французский', 'испанский', 'арабский', 'польский',
                          'датский', 'Турецкий', 'нидерландский', 'китайский (упрощ. письмо)'],

    "genres": ['Боевик', 'Приключения', 'Гонки', 'Аркада', 'Ужасы', 'Пазлы',
                       'Симуляторы', 'Спорт', 'Семейные', 'Казуальные',
                       'Ролевые игры', 'Тусовка', 'Стратегия',
                       'Единоборства', 'Шутер', 'MUSIC/RHYTHM', 'Уникальные']
}

SELECTORS = {
    "lp full": {'class': 'paginator-control__end'},
    "lp new games": {'class': 'paginator-control__end'},
    "collect ng": {'href': re.compile(r'\/ru-ru\/product\/.*')},
    "collect full": {'href': re.compile(r'\/ru-ru\/product\/.*')},
    "ps plus container": {'class': 'grid-cell--game'},
    "ps plus link": {'class': 'internal-app-link'},
}

GAME_SELECTORS = {
    "cover": "div.large-3 > div.pdp__"
             "thumbnail-img > div.product-image > "
             "div.product-image__img > "
             "div.product-image__img--main > img",
    "title": {"class": "pdp__title"},
    "publisher": {"class": "provider-info__text"},
    "category": {"class": "provider-info__list-item"},
    "price": {"class": "price-display__price"},
    "previous price":  {"class": "price-display__strikethrough"},
    "psplus discount": {"class": "price-display__price__label"},
    "specs": {"class": "tech-specs__pivot-menus"},
    "description": {"class": "pdp__description"},
}


PATTERNS = {
    "lp full": re.compile('\/(\d{3})\?'),
    "lp new games": re.compile('\/(\d{1})\?'),
    "lp discounts": re.compile('\/(\d{2})\?'),
    "lp top sellers": re.compile('\/(\d{1})'),
}



from bs4 import BeautifulSoup
import requests


class Helpers:

    @staticmethod
    def get_soup(link):
        response = requests.get(link)
        source = response.content
        soup = BeautifulSoup(source, features='html.parser')
        return soup



import threading
from .utils import Helpers
from .variables import EXTERNAL, SELECTORS, PATTERNS


class PS4StoreRussia:

    def __init__(self):
        self.store_homepage = EXTERNAL["store_homepage"]
        self.new_games_homepage = EXTERNAL["new_games_homepage"]
        self.links = set()
        self.discounts = EXTERNAL["discounts"]
        self.tops = EXTERNAL["top sellers home"]
        self.f2p = EXTERNAL["ftp home"]
        self.soon = EXTERNAL["soon home"]
        self.discount_games = set()
        self.new_games_links = set()
        self.threads = []
        self.ps_plus_games = []
        self.top_sellers = set()
        self.f2p_games = set()
        self.soon_tbr_games = set()

    def __get_soon_tbr_last_page_number(self):
        soup = Helpers.get_soup(self.soon)
        end = soup.find('a', SELECTORS["lp new games"])
        try:
            result = PATTERNS["lp top sellers"].search(end['href'])
        except TypeError:
            return False
        else:
            return int(result[1])

    def __get_free_to_play_last_page_number(self):
        soup = Helpers.get_soup(self.f2p)
        end = soup.find('a', SELECTORS["lp new games"])
        result = PATTERNS["lp top sellers"].search(end['href'])
        return int(result[1])

    def __get_top_sellers_catalogue_last_page_number(self):
        soup = Helpers.get_soup(self.tops)
        end = soup.find('a', SELECTORS["lp new games"])
        result = PATTERNS["lp top sellers"].search(end['href'])
        return int(result[1])

    def __get_discounts_catalogue_last_page_number(self):
        soup = Helpers.get_soup(self.discounts)
        end = soup.find('a', SELECTORS["lp new games"])
        result = PATTERNS["lp discounts"].search(end['href'])
        return int(result[1])

    def __collect_soon_tbr_games(self):
        lp = self.__get_soon_tbr_last_page_number()
        if lp:
            for i in range(1, lp + 1):
                soup = Helpers.get_soup(EXTERNAL["soon"] % i)
                for link in soup.find_all('a', SELECTORS["collect ng"]):
                    self.soon_tbr_games.add(EXTERNAL["host"] + link['href'])
        else:
            soup = Helpers.get_soup(self.soon)
            for link in soup.find_all('a', SELECTORS["collect ng"]):
                self.soon_tbr_games.add(EXTERNAL["host"] + link['href'])

    def __collect_f2p_games_links(self):
        for i in range(1, self.__get_free_to_play_last_page_number() + 1):
            soup = Helpers.get_soup(EXTERNAL["ftp"] % i)
            for link in soup.find_all('a', SELECTORS["collect ng"]):
                self.f2p_games.add(EXTERNAL["host"] + link['href'])

    def __collect_top_sellers_links(self):
        for i in range(1, self.__get_top_sellers_catalogue_last_page_number() + 1):
            soup = Helpers.get_soup(EXTERNAL["top sellers"] % i)
            for link in soup.find_all('a', SELECTORS["collect ng"]):
                self.top_sellers.add(EXTERNAL["host"] + link['href'])

    def __collect_discounts_links(self):
        for i in range(1, self.__get_discounts_catalogue_last_page_number() + 1):
            soup = Helpers.get_soup(EXTERNAL["discounts homepage"] % i)
            for link in soup.find_all('a', SELECTORS["collect ng"]):
                self.discount_games.add(EXTERNAL["host"] + link['href'])

    def get_all_discounts_links(self):
        self.__collect_discounts_links()
        return self.discount_games

    def __get_full_catalogue_last_page_number(self):
        soup = Helpers.get_soup(self.store_homepage)
        end = soup.find('a', SELECTORS["lp full"])
        result = PATTERNS["lp full"].search(end['href'])
        return int(result[1])

    def __get_new_games_last_page_number(self):
        soup = Helpers.get_soup(self.new_games_homepage)
        end = soup.find('a', SELECTORS["lp new games"])
        result = PATTERNS["lp new games"].search(end['href'])
        return int(result[1])

    def __collect_new_games_links(self):
        for i in range(1, self.__get_new_games_last_page_number() + 1):
            soup = Helpers.get_soup(EXTERNAL["latest"] % i)
            for link in soup.find_all('a', SELECTORS["collect ng"]):
                self.new_games_links.add(EXTERNAL["host"] + link['href'])

    def __collect_all_catalogue_links(self, number):
        soup = Helpers.get_soup(EXTERNAL["all"] % number)
        for link in soup.find_all('a', SELECTORS["collect full"]):
            self.links.add(EXTERNAL["host"] + link['href'])

    def __run_threads(self):
        for i in range(1, self.__get_full_catalogue_last_page_number() + 1):
            worker = threading.Thread(target=self.__collect_all_catalogue_links, args=(i,))
            self.threads.append(worker)
        for thread in self.threads:
            thread.start()
        for thread in self.threads:
            thread.join()

    def get_ps_plus_deals(self):
        soup = Helpers.get_soup(EXTERNAL["ps plus"])
        divs = soup.find_all('div', SELECTORS["ps plus container"])
        for container in divs:
            self.ps_plus_games.append(EXTERNAL["host"] +
                                      container.find('a', SELECTORS["ps plus link"])['href'])
        return self.ps_plus_games

    def get_all_links(self):
        self.__run_threads()
        return self.links

    def get_all_new_games_links(self):
        self.__collect_new_games_links()
        return self.new_games_links

    def get_top_sellers(self):
        self.__collect_top_sellers_links()
        return self.top_sellers

    def get_soon_tbr_games(self):
        self.__collect_soon_tbr_games()
        return self.soon_tbr_games

    def get_f2p_games(self):
        self.__collect_f2p_games_links()
        return self.f2p_games


print(
    PS4StoreRussia().get_f2p_games()
)



import requests
import re
import json
import yaml
from .variables import SPECS, GAME_SELECTORS, EXTERNAL
from bs4 import BeautifulSoup


class PS4Game:
    def __init__(self, url: str = None, alias: str = None):
        """
        :param url: https://store.playstation.com/ru-ru/product/EP0002-CUSA23470_00-CB4STANDARD00001
        :param alias: EP0002-CUSA23470_00-CB4STANDARD00001
        """
        self.url = url if url else f"{EXTERNAL['product']}{alias}"
        self.languages = SPECS["languages"]
        self.genres = SPECS["genres"]
        self.soup = None
        self.specs = []

    def __get_title(self):
        return self.soup.find("h2", GAME_SELECTORS["title"]).text

    def __get_publisher(self):
        try:
            publisher = self.soup.find("h5", GAME_SELECTORS["publisher"]).text
        except AttributeError:
            return ""
        else:
            return "" if '\n' in publisher else publisher

    def __get_category(self):
        try:
            return self.soup.find("span", GAME_SELECTORS["category"]).text
        except AttributeError:
            return ""

    def __get_price(self):
        try:
            return int(re.sub(r'\D', '', self.soup.find("h3", GAME_SELECTORS["price"]).text))
        except AttributeError:
            return 0

    def __get_former_price(self):
        try:
            return int(re.sub(r'\D', '', self.soup.find("span", GAME_SELECTORS["previous price"]).text))
        except AttributeError:
            return 0

    def __get_ps_plus(self):
        try:
            return self.soup.find("div", GAME_SELECTORS["psplus discount"]).text
        except AttributeError:
            return ""

    def __get_cover(self):
        cover_picture_url = self.soup.select(GAME_SELECTORS["cover"])
        return cover_picture_url[0]['src'] or ""

    def __get_description(self):
        try:
            return self.soup.find("div", GAME_SELECTORS["description"]).text
        except AttributeError:
            return ""

    def __get_language(self):
        return list(set(filter(lambda l: l in self.languages, self.specs)))

    def __get_genre(self):
        return list(set(filter(lambda g: g in self.genres, self.specs)))

    def __get_size(self):
        return " ".join(self.specs[self.specs.index('Размер файла'):]) or None

    def __get_age(self):
        img_tag = self.soup.find('img', {'class': 'content-rating__rating-img'})
        src_attribute = re.search(r'/(\d+)\.(png|jpg|svg)', img_tag['src'])
        return int(src_attribute.group(1)) or 0

    def __get_screenshots(self):
        if self.__get_cover():
            api_url = re.sub(r'\/image\?.*', '', self.__get_cover())
            json_data = requests.get(api_url).json()
            return [element["url"] for element in json_data["mediaList"]["screenshots"]] or []

    def __load_page(self):
        response = requests.get(self.url.encode("utf-8").decode("utf-8-sig"))
        html = response.content
        self.soup = BeautifulSoup(html, features='html.parser')
        self.specs = list(map(lambda x: x.strip(), self.soup.find("div", GAME_SELECTORS["specs"]).text.split('\n')))

    def __make_payload(self):
        """
        Returns a dict of game specs
        """
        self.__load_page()

        return {
            "title": self.__get_title(),
            "link": self.url,
            "details": {
                "publisher": self.__get_publisher(),
                "category": self.__get_category(),
                "price": self.__get_price(),
                "previous price": self.__get_former_price(),
                "on sale": True if self.__get_former_price() else False,
                "PS Plus discount": self.__get_ps_plus(),
                "size": self.__get_size(),
                "age limit": self.__get_age(),
                "description": self.__get_description(),
                "cover": self.__get_cover(),
                "language": self.__get_language(),
                "genre": self.__get_genre(),
                "screenshots": self.__get_screenshots(),
            }
        }

    def as_yaml(self):
        """
        Return game info as YAML
        """
        return yaml.dump([self.__make_payload()], allow_unicode=True)

    def as_xml(self):
        """
        Return game info as XML
        """
        pass

    def as_json(self):
        """
        Return game info as JSON string
        """
        return json.dumps(self.__make_payload(), ensure_ascii=False, indent=4)